package guest;
 
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.TypedQuery;
import javax.persistence.NamedQuery;
import javax.persistence.NamedQueries;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;



@NamedQueries( {
	@NamedQuery(
	        name="Guest.getAll",
	        query="select g from guest.Guest g order by g.name"
	) } )
@Stateless
@TransactionManagement(TransactionManagementType.CONTAINER)
@TransactionAttribute(TransactionAttributeType.REQUIRED)
public class GuestDao {
    // Injected database connection:
	@PersistenceContext(unitName = "guest-pu") private EntityManager em;

    // Stores a new guest
    public void persist(final Guest guest) {
		withinTransaction( "persist" );
		System.out.println( "will persist : "  + guest + "...."  );
        this.em.persist(guest);
		this.em.flush();	
		System.out.println( "persisted: "  + guest );
		findAllGuests();
    }
	
    // Retrieves all the guests:
    public List<Guest> getAllGuests() {
		return findAllGuests();
    }
	
	private void withinTransaction(final String methodName ) {
		javax.persistence.EntityTransaction et = em.getTransaction();
		if ( et.isActive() ) 
			System.out.println( methodName + " is within an active transaction" );
		else 
			System.out.println( methodName + " is out of an active transaction" );
	}
	
	private List<Guest> findAllGuests() {
		withinTransaction("findAllGuests");
		javax.persistence.EntityTransaction et = em.getTransaction();

		javax.persistence.metamodel.ManagedType<Guest> type = em.getMetamodel().managedType(Guest.class);
		if ( type == null ) 				System.out.println( "ManagedType Guest not found"  );

		// NamedQuery does not work
		// TypedQuery<Guest> query = this.em.createNamedQuery( "Guest.getAll", Guest.class );

		TypedQuery<Guest> query = this.em.createQuery( "select g from guest.Guest g order by g.name", Guest.class);

		final List<Guest> guests =  query.getResultList();
		System.out.println( "persisted Guest count: " + guests.size() );

		return guests;
    }
}